"""
Tools for development for end user:
Turbo: 2 or 3 line NN class set up; creates a appropriate NN based on data size and structure.
Format-Data: Formats data for NN to use (using pyndb)
Img-Matrix: converts a image to a 3d convolution matrix (options for RGB and Grayscale)
HelpIHaveNoIdeaWhatImDoing: documentation and a start-up guide
Extract-Verification-Data: extracts verification data from a CSV file (more data formats soon)
"""




from pyndb import PYNDatabase
from deeprai.engine.base_layer import VelocityVals, WeightVals, NeuronVals, ActivationList, \
    ActivationDerivativeList, DerivativeVals, Optimizer, LossString, LayerVals
from deeprai.engine.build_model import Build
import numpy as np
class Load:
    def __init__(self, file_location):
        builder = Build()
        self.format_file(file_location)
        self.db = PYNDatabase(file_location, filetype='pickled')
        VelocityVals.Velocities = self.db.Velocity.val
        WeightVals.Weights = self.db.Weight.val
        LayerVals.Layers = self.db.Layer.val
        NeuronVals.Neurons = self.db.Neuron.val
        DerivativeVals.Derivatives = self.db.Derivative.val
        LossString = self.db.Loss.val
        for activation, derivative in zip(self.db.Activation.val[1:], self.db.ActivationDerivative.val[1:]):
            builder.convert_activations(activation)
            builder.convert_derivatives(derivative)
        builder.convert_loss(self.db.Loss.val)




    def format_file(self, file_location):
        file_ext = file_location.split(".")
        if file_ext[-1] != "deepr":
            raise Exception(f"Can not import that file extension, use .deepr")
            return



from pyndb import PYNDatabase
from deeprai.engine.base_layer import VelocityVals, WeightVals, ActivationListString, \
    ActivationDerivativeListString, DerivativeVals, OptimizerString, LossString, LayerVals, NeuronVals
class Save:
    def __init__(self, file_location):
        file_location = self.format_file(file_location)
        self.db = PYNDatabase(file_location, filetype='pickled')
        self.FileLocation = file_location
        self.Velocity = VelocityVals.Velocities
        self.Weight = WeightVals.Weights
        self.Layer = LayerVals.Layers
        self.Derivative = DerivativeVals.Derivatives
        self.Activation = ActivationListString
        self.ActivationDerivative = ActivationDerivativeListString
        self.Neuron = NeuronVals.Neurons
        # self.Optimizer = OptimizerString
        self.Loss = LossString

    def save(self):
        self.db.set("Velocity",self.Velocity)
        self.db.set("Weight", self.Weight)
        self.db.set("Layer", self.Layer)
        self.db.set("Derivative", self.Derivative)
        self.db.set("Activation", self.Activation)
        self.db.set("ActivationDerivative", self.ActivationDerivative)
        # self.db.set("Optimizer", self.Optimizer)
        self.db.set("Neuron", self.Neuron)
        self.db.set("Loss", self.Loss)
        self.db.save(self.FileLocation)

    def format_file(self, file_location):
        file_ext = file_location.split(".")
        if len(file_ext) == 1:
            raise Exception(f"Failed to specify file extension, try '{file_location}.deepr?' ")
            return
        file_ext[-1] = "deepr"
        return '.'.join(file_ext)




class Convolutional:
    def __init__(self):
        self.layers = []
        self.kernels = []
        self.bias = []
        self.padding = None
        self.strides = None
        self.output_labels = None

    def add_kernel(self, amount, shape, max_size: int = 2, ):
        self.stack_layer.create_kernel(amount, shape, max_size)

    def add_pool(self, shape):
        self.stack_layer.create_pool()

    def add_dense(self, neurons, activation=''):
        self.stack_layer.create_dense()

    def flatten(self):
        self.stack_layer.create_flat()

    def merge(self, merge_type='sum'):
        self.stack_layer.create_merge()

    def model_optimizers(self, optimizer='', loss=''):
        pass

    def train_model(self, input_data, verify_data, batch_size=10, epoches=500, learning_rate=0.1, momentum=0.6, verbose=True):
        pass

    def summery(self):
        pass


from colorama import Fore, Style
import os
for file in os.listdir('.'):
    if file.endswith('.so') or file.endswith('.pyd'):
        src = os.path.join(os.getcwd(), file)
        dst = os.path.join(os.getcwd(), 'deeprai/engine/cython', file)
        os.rename(src, dst)
print(Fore.CYAN + Style.BRIGHT + "Loaded DeeprAI 0.0.8 BETA")
print(Fore.RESET + Style.NORMAL)



from deeprai.engine.cython import activation as act
class Relu:
    def relu(self, neuron):
        return act.relu(neuron)

    def relu_derivative(self, relu):
        return act.relu_derivative(relu)





import sys
from setuptools import setup
from setuptools import find_packages
from setuptools.extension import Extension
import shutil
import os
try:
    from Cython.Build import cythonize
except ImportError:
    cythonize = None

with open("README.md", "r") as f:
    long_description = f.read()

def no_cythonize(extensions, **_ignore):
    for extension in extensions:
        sources = []
        for sfile in extension.sources:
            path, ext = os.path.splitext(sfile)
            if ext in (".pyx", ".py"):
                if extension.language == "c++":
                    ext = ".cpp"
                else:
                    ext = ".c"
                sfile = path + ext
            sources.append(sfile)
        extension.sources[:] = sources
    return extensions


extensions = [
    Extension("deeprai/engine/cython/activation", ["deeprai/engine/cython/activation.c"]),
    Extension("deeprai/engine/cython/dense_operations", ["deeprai/engine/cython/dense_operations.c"]),
    Extension("deeprai/engine/cython/dense_train_loop", ["deeprai/engine/cython/dense_train_loop.c"]),
    Extension("deeprai/engine/cython/loss",["deeprai/engine/cython/loss.c"]),
    Extension("deeprai/engine/cython/optimizers", ["deeprai/engine/cython/optimizers.c"]),
    ]
CYTHONIZE = bool(int(os.getenv("CYTHONIZE", 0))) and cythonize is not None

if CYTHONIZE:
    compiler_directives = {"language_level": 3, "embedsignature": True}
    extensions = cythonize(extensions, compiler_directives=compiler_directives)
else:
    extensions = no_cythonize(extensions)

setup(
    name='deeprai',
    version='0.0.8',
    author='Kieran Carter',
    description='A easy to use and beginner friendly neural network tool box that anyone can pick up and explorer machine learning! ',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Deepr-ai/Deepr-ai",
    packages=find_packages(),
    ext_modules=extensions,
    install_requires=[
        'cython',
        'numpy',
        'pyndb',
        'alive_progress',
        'colorama',
        'cryptography'
    ],
classifiers=["Programming Language :: Python :: 3",
             "Programming Language :: Cython",
             "License :: OSI Approved :: Apache Software License",
             "Operating System :: OS Independent",]

)
for file in os.listdir('.'):
    if file.endswith('.so') or file.endswith('.pyd'):
        src = os.path.join(os.getcwd(), file)
        dst = os.path.join(os.getcwd(), 'deeprai/engine/cython', file)
        os.rename(src, dst)